"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = __importDefault(require("child_process"));
const util_1 = __importDefault(require("util"));
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
const shell = util_1.default.promisify(child_process_1.default.exec);
ava_1.default.beforeEach(async () => {
    await fs_extra_1.default.remove(`${CWD}/memserver`);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.afterEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.after.always(async () => {
    await fs_extra_1.default.remove(`${CWD}/memserver`);
});
ava_1.default.serial("$ memserver g fixtures | without memserver directory raises error", async (t) => {
    t.plan(3);
    const folderExistence = await fs_extra_1.default.pathExists(`${CWD}/memserver`);
    t.true(!folderExistence);
    let result = await shell(`node ${CWD}/dist/cli.js generate fixtures`);
    t.true(result.stdout.includes("[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?"));
    result = await shell(`node ${CWD}/dist/cli.js g fixtures`);
    t.true(result.stdout.includes("[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?"));
});
ava_1.default.serial("$ memserver g fixtures [modelName] | without memserver directory raises error", async (t) => {
    t.plan(3);
    const folderExistence = await fs_extra_1.default.pathExists(`${CWD}/memserver`);
    t.true(!folderExistence);
    let result = await shell(`node ${CWD}/dist/cli.js generate fixtures user`);
    t.true(result.stdout.includes("[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?"));
    result = await shell(`node ${CWD}/dist/cli.js g fixtures`);
    t.true(result.stdout.includes("[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?"));
});
ava_1.default.serial("$ memserver g fixtures | works for the entire state", async (t) => {
    t.plan(9);
    await generateMemServerState();
    const { stdout } = await shell(`ts-node ${CWD}/dist/cli.js g fixtures`);
    t.true(stdout.includes("[MemServer] data written to /fixtures/users.ts"));
    t.true(stdout.includes("[MemServer] data written to /fixtures/likes.ts"));
    t.true(stdout.includes("[MemServer] data written to /fixtures/photos.ts"));
    t.true(stdout.includes("[MemServer] data written to /fixtures/photo-comments.ts"));
    const fixturesPath = `${CWD}/memserver/fixtures`;
    t.deepEqual(await fs_extra_1.default.readdir(fixturesPath), [
        "likes.ts",
        "photo-comments.ts",
        "photos.ts",
        "users.ts"
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/users.ts`)))).default, [
        {
            password: "123456",
            authentication_token: "12or12rnfasdfzlemfp1m3epfm134",
            id: 1,
            first_name: "Izel",
            last_name: "Nakri"
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photos.ts`)))).default, [
        { id: 1, name: "Me skiing", href: "ski-trip.jpeg", is_public: false },
        { id: 2, name: "Family photo", href: "family-photo.jpeg", is_public: true },
        { id: 3, name: "Selfie", href: "selfie.jpeg", is_public: false },
        { id: 4, name: "Travel photo", href: "travel-photo.jpeg", is_public: false }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photo-comments.ts`)))).default, [
        {
            uuid: "499ec646-493f-4eea-b92e-e383d94182f4",
            content: "What a nice photo!",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "77653ad3-47e4-4ec2-b49f-57ea36a627e7",
            content: "I agree",
            photo_id: 1,
            user_id: 2
        },
        {
            uuid: "d351963d-e725-4092-a37c-1ca1823b57d3",
            content: "I was kidding",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525f5f29",
            content: "Interesting indeed",
            photo_id: 2,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525faaaa",
            content: "This is badass",
            photo_id: 1,
            user_id: 1
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/likes.ts`)))).default, []);
});
ava_1.default.serial("$ memserver generate fixtures [modelName] works", async (t) => {
    t.plan(12);
    await generateMemServerState();
    const { stdout } = await shell(`ts-node ${CWD}/dist/cli.js generate fixtures users`);
    t.true(stdout.includes("[MemServer] data written to /fixtures/users.ts"));
    const fixturesPath = `${CWD}/memserver/fixtures`;
    t.deepEqual(await fs_extra_1.default.readdir(fixturesPath), ["photo-comments.ts", "photos.ts", "users.ts"]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/users.ts`)))).default, [
        {
            password: "123456",
            authentication_token: "12or12rnfasdfzlemfp1m3epfm134",
            id: 1,
            first_name: "Izel",
            last_name: "Nakri"
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photos.ts`)))).default, [
        {
            id: 1,
            name: "Ski trip",
            href: "ski-trip.jpeg",
            is_public: false
        },
        {
            id: 2,
            name: "Family photo",
            href: "family-photo.jpeg",
            is_public: true
        },
        {
            id: 3,
            name: "Selfie",
            href: "selfie.jpeg",
            is_public: false
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photo-comments.ts`)))).default, [
        {
            uuid: "499ec646-493f-4eea-b92e-e383d94182f4",
            content: "What a nice photo!",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "77653ad3-47e4-4ec2-b49f-57ea36a627e7",
            content: "I agree",
            photo_id: 1,
            user_id: 2
        },
        {
            uuid: "d351963d-e725-4092-a37c-1ca1823b57d3",
            content: "I was kidding",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525f5f29",
            content: "Interesting indeed",
            photo_id: 2,
            user_id: 1
        }
    ]);
    t.true(!(await fs_extra_1.default.pathExists(`${fixturesPath}/likes.js`)));
    let result = await shell(`ts-node ${process.cwd()}/dist/cli.js generate fixtures photos`);
    t.true(result.stdout.includes("[MemServer] data written to /fixtures/photos.ts"));
    t.deepEqual(await fs_extra_1.default.readdir(fixturesPath), ["photo-comments.ts", "photos.ts", "users.ts"]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/users.ts`)))).default, [
        {
            password: "123456",
            authentication_token: "12or12rnfasdfzlemfp1m3epfm134",
            id: 1,
            first_name: "Izel",
            last_name: "Nakri"
        }
    ]);
    const photosFixtureData = (await fs_extra_1.default.readFile(`${fixturesPath}/photos.ts`)).toString();
    t.deepEqual(eval(photosFixtureData.slice(15, photosFixtureData.length - 1)), [
        // TODO: babel cache: false doesnt work
        { id: 1, name: "Me skiing", href: "ski-trip.jpeg", is_public: false },
        { id: 2, name: "Family photo", href: "family-photo.jpeg", is_public: true },
        { id: 3, name: "Selfie", href: "selfie.jpeg", is_public: false },
        { id: 4, name: "Travel photo", href: "travel-photo.jpeg", is_public: false }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photo-comments.ts`)))).default, [
        {
            uuid: "499ec646-493f-4eea-b92e-e383d94182f4",
            content: "What a nice photo!",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "77653ad3-47e4-4ec2-b49f-57ea36a627e7",
            content: "I agree",
            photo_id: 1,
            user_id: 2
        },
        {
            uuid: "d351963d-e725-4092-a37c-1ca1823b57d3",
            content: "I was kidding",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525f5f29",
            content: "Interesting indeed",
            photo_id: 2,
            user_id: 1
        }
    ]);
    t.true(!(await fs_extra_1.default.pathExists(`${fixturesPath}/likes.ts`)));
});
ava_1.default.serial("$ memserver g fixtures [modelName] works", async (t) => {
    t.plan(12);
    await generateMemServerState();
    const { stdout } = await shell(`ts-node ${CWD}/dist/cli.js g fixtures users`);
    t.true(stdout.includes("[MemServer] data written to /fixtures/users.ts"));
    const fixturesPath = `${CWD}/memserver/fixtures`;
    t.deepEqual(await fs_extra_1.default.readdir(fixturesPath), ["photo-comments.ts", "photos.ts", "users.ts"]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/users.ts`)))).default, [
        {
            password: "123456",
            authentication_token: "12or12rnfasdfzlemfp1m3epfm134",
            id: 1,
            first_name: "Izel",
            last_name: "Nakri"
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photos.ts`)))).default, [
        {
            id: 1,
            name: "Ski trip",
            href: "ski-trip.jpeg",
            is_public: false
        },
        {
            id: 2,
            name: "Family photo",
            href: "family-photo.jpeg",
            is_public: true
        },
        {
            id: 3,
            name: "Selfie",
            href: "selfie.jpeg",
            is_public: false
        }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photo-comments.ts`)))).default, [
        {
            uuid: "499ec646-493f-4eea-b92e-e383d94182f4",
            content: "What a nice photo!",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "77653ad3-47e4-4ec2-b49f-57ea36a627e7",
            content: "I agree",
            photo_id: 1,
            user_id: 2
        },
        {
            uuid: "d351963d-e725-4092-a37c-1ca1823b57d3",
            content: "I was kidding",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525f5f29",
            content: "Interesting indeed",
            photo_id: 2,
            user_id: 1
        }
    ]);
    t.true(!(await fs_extra_1.default.pathExists(`${fixturesPath}/likes.ts`)));
    let result = await shell(`ts-node ${CWD}/dist/cli.js g fixtures photos`);
    t.true(result.stdout.includes("[MemServer] data written to /fixtures/photos.ts"));
    t.deepEqual(await fs_extra_1.default.readdir(fixturesPath), ["photo-comments.ts", "photos.ts", "users.ts"]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/users.ts`)))).default, [
        {
            password: "123456",
            authentication_token: "12or12rnfasdfzlemfp1m3epfm134",
            id: 1,
            first_name: "Izel",
            last_name: "Nakri"
        }
    ]);
    const photosFixtureData = (await fs_extra_1.default.readFile(`${fixturesPath}/photos.ts`)).toString();
    t.deepEqual(eval(photosFixtureData.slice(15, photosFixtureData.length - 1)), [
        // TODO: babel cache: false doesnt work
        { id: 1, name: "Me skiing", href: "ski-trip.jpeg", is_public: false },
        { id: 2, name: "Family photo", href: "family-photo.jpeg", is_public: true },
        { id: 3, name: "Selfie", href: "selfie.jpeg", is_public: false },
        { id: 4, name: "Travel photo", href: "travel-photo.jpeg", is_public: false }
    ]);
    t.deepEqual((await Promise.resolve().then(() => __importStar(require(`${fixturesPath}/photo-comments.ts`)))).default, [
        {
            uuid: "499ec646-493f-4eea-b92e-e383d94182f4",
            content: "What a nice photo!",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "77653ad3-47e4-4ec2-b49f-57ea36a627e7",
            content: "I agree",
            photo_id: 1,
            user_id: 2
        },
        {
            uuid: "d351963d-e725-4092-a37c-1ca1823b57d3",
            content: "I was kidding",
            photo_id: 1,
            user_id: 1
        },
        {
            uuid: "374c7f4a-85d6-429a-bf2a-0719525f5f29",
            content: "Interesting indeed",
            photo_id: 2,
            user_id: 1
        }
    ]);
    t.true(!(await fs_extra_1.default.pathExists(`${fixturesPath}/likes.js`)));
});
// test('$ memserver generate fixtures [modelName] with wrong modelName raises', (t) => {
// });
// test('$ memserver g fixtures [modelName] with wrong modelName raises', (t) => {
// });
function generateMemServerState() {
    return new Promise(async (resolve) => {
        const modelFileContent = (className) => `import Model from '${CWD}/dist/model';
    export default class ${className} extends Model{
    }`;
        await fs_extra_1.default.mkdir(`${CWD}/memserver`);
        await Promise.all([fs_extra_1.default.mkdir(`${CWD}/memserver/models`), fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)]);
        await Promise.all([
            fs_extra_1.default.writeFile(`${CWD}/memserver/models/user.ts`, `import Model from '${CWD}/dist/model';
      export default class User extends Model {
        static defaultAttributes = {
          password: '123456',
          authentication_token() {
            return '12or12rnfasdfzlemfp1m3epfm134';
          }
        }
      }`),
            fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, modelFileContent("Photo")),
            fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, `import Model from '${CWD}/dist/model';
      export default class PhotoComment extends Model {
        static createForUser(user, photo, photoOptions) {
          console.log(user);
          console.log(photo);
          return this.insert(Object.assign({}, photoOptions, {
            photo_id: photo.id, user_id: user.id
          }));
        }
      }`),
            fs_extra_1.default.writeFile(`${CWD}/memserver/models/likes.ts`, modelFileContent("Likes")),
            fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, "export default function() {}"),
            fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
        {
          id: 1,
          name: 'Ski trip',
          href: 'ski-trip.jpeg',
          is_public: false
        },
        {
          id: 2,
          name: 'Family photo',
          href: 'family-photo.jpeg',
          is_public: true
        },
        {
          id: 3,
          name: 'Selfie',
          href: 'selfie.jpeg',
          is_public: false
        }
      ];`),
            fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photo-comments.ts`, `export default [
        {
          uuid: '499ec646-493f-4eea-b92e-e383d94182f4',
          content: 'What a nice photo!',
          photo_id: 1,
          user_id: 1
        },
        {
          uuid: '77653ad3-47e4-4ec2-b49f-57ea36a627e7',
          content: 'I agree',
          photo_id: 1,
          user_id: 2
        },
        {
          uuid: 'd351963d-e725-4092-a37c-1ca1823b57d3',
          content: 'I was kidding',
          photo_id: 1,
          user_id: 1
        },
        {
          uuid: '374c7f4a-85d6-429a-bf2a-0719525f5f29',
          content: 'Interesting indeed',
          photo_id: 2,
          user_id: 1
        }
      ];`),
            fs_extra_1.default.writeFile(`${CWD}/memserver/initializer.ts`, `
        import User from './models/user';
        import Photo from './models/photo';
        import PhotoComment from './models/photo-comment';
        import photos from './fixtures/photos';
        import photoComments from './fixtures/photo-comments';

        export default function() {
          Photo.resetDatabase(photos);
          PhotoComment.resetDatabase(photoComments);

          const user = User.insert({ first_name: 'Izel', last_name: 'Nakri' });
          const photo = Photo.find(1);

          Photo.insert({ name: 'Travel photo', href: 'travel-photo.jpeg', is_public: false });
          PhotoComment.createForUser(user, photo, {
            uuid: '374c7f4a-85d6-429a-bf2a-0719525faaaa', content: 'This is badass'
          });

          Photo.update({ id: 1, name: 'Me skiing' });
      }`)
        ]);
        resolve();
    });
}
