"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const child_process_1 = __importDefault(require("child_process"));
const CWD = process.cwd();
const shell = util_1.default.promisify(child_process_1.default.exec);
ava_1.default.beforeEach(async () => {
    await fs_extra_1.default.remove(`${CWD}/memserver`);
});
ava_1.default.serial("$ memserver g | and $ memserver generate | without memserver directory raises", async (t) => {
    t.plan(2);
    const { stdout } = await shell(`ts-node ${CWD}/dist/cli.js generate`);
    t.is(stdout, "[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?\n");
    let result = await shell(`ts-node ${CWD}/dist/cli.js g`);
    t.is(result.stdout, "[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?\n");
});
ava_1.default.serial("$ memserver g model | and $ memserver generate model | without memserver directory raises", async (t) => {
    t.plan(2);
    const { stdout } = await shell(`ts-node ${CWD}/dist/cli.js generate model`);
    t.is(stdout, "[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?\n");
    let result = await shell(`ts-node ${CWD}/dist/cli.js g model`);
    t.is(result.stdout, "[Memserver CLI] cannot find /memserver folder. Did you run $ memserver init ?\n");
});
ava_1.default.serial("$ memserver g model | and $ memserver generate model | without model raises", async (t) => {
    t.plan(1);
    await initializeMemServer();
    const { stdout } = await shell(`node ${CWD}/dist/cli.js generate model`);
    t.is(stdout, "[Memserver CLI] Please put a modelName to the memserver generate. Example: $ memserver generate model user\n");
});
const EXPECTED_OUTPUT = "[Memserver CLI] /memserver/models/email.ts created\n" +
    "[Memserver CLI] /memserver/fixtures/emails.ts created\n";
ava_1.default.serial("$ memserver g [modelName] | works", async (t) => {
    t.plan(5);
    await initializeMemServer();
    const [emailModelExists, emailFixturesBuffer] = await Promise.all([
        fs_extra_1.default.pathExists(`${CWD}/memserver/models/email.ts`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/fixtures/emails.ts`)
    ]);
    t.true(!emailModelExists);
    t.true(!emailFixturesBuffer);
    const { stdout } = await shell(`node ${CWD}/dist/cli.js g model email`);
    t.is(stdout, EXPECTED_OUTPUT);
    const [emailModelBuffer, userFixturesBuffer] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver/models/email.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/fixtures/emails.ts`)
    ]);
    t.is(emailModelBuffer.toString(), `import Model from 'memserver/model';

export default class Email extends Model {
  constructor() {
    super();
  }
}`);
    t.is(userFixturesBuffer.toString(), "export default [\n];");
});
ava_1.default.serial("$ memserver generate [modelName] | works", async (t) => {
    t.plan(5);
    await initializeMemServer();
    const [emailModelExists, emailFixturesBuffer] = await Promise.all([
        fs_extra_1.default.pathExists(`${CWD}/memserver/models/user.ts`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/fixtures/users.ts`)
    ]);
    t.true(!emailModelExists);
    t.true(!emailFixturesBuffer);
    const { stdout } = await shell(`node ${process.cwd()}/dist/cli.js generate model email`);
    t.is(stdout, EXPECTED_OUTPUT);
    const [emailModelBuffer, userFixturesBuffer] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver/models/email.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/fixtures/emails.ts`)
    ]);
    t.is(emailModelBuffer.toString(), `import Model from 'memserver/model';

export default class Email extends Model {
  constructor() {
    super();
  }
}`);
    t.is(userFixturesBuffer.toString(), "export default [\n];");
});
function initializeMemServer() {
    return new Promise(async (resolve) => {
        if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
            await fs_extra_1.default.remove(`${CWD}/memserver`);
        }
        const memServerDirectory = `${CWD}/memserver`;
        await fs_extra_1.default.mkdir(memServerDirectory);
        await Promise.all([
            fs_extra_1.default.writeFile(`${memServerDirectory}/server.js`, `export default function(Models) {
    }`),
            fs_extra_1.default.writeFile(`${memServerDirectory}/initializer.js`, `export default function(Models) {
      }`),
            fs_extra_1.default.mkdir(`${memServerDirectory}/fixtures`),
            fs_extra_1.default.mkdir(`${memServerDirectory}/models`)
        ]);
        resolve();
    });
}
