"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const util_1 = __importDefault(require("util"));
const child_process_1 = __importDefault(require("child_process"));
const CWD = process.cwd();
const shell = util_1.default.promisify(child_process_1.default.exec);
ava_1.default.afterEach.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("$ memserver | and $ memserver helper | and $ memserver h | without arguments shows help screen", async (t) => {
    t.plan(3);
    const jsonDataBuffer = await fs_extra_1.default.readFile(`${CWD}/package.json`);
    const version = JSON.parse(jsonDataBuffer.toString()).version;
    const expectedOutput = `[Memserver CLI v${version}] Usage: memserver <command (Default: help)>

memserver init | new                    # Sets up the initial memserver folder structure
memserver generate model [ModelName]    # Generates the initial files for a MemServer Model [alias: "memserver g model"]
memserver generate fixtures             # Outputs your initial MemServer state as pure javascript fixture files
memserver generate fixtures [ModelName] # Outputs your initial MemServer state for certain model as pure javascript fixture
memserver console                       # Starts a MemServer console in node.js [alias: "memserver c"]
`;
    let result = await shell(`node ${CWD}/dist/cli.js`);
    t.true(result.stdout.includes(expectedOutput));
    result = await shell(`node ${CWD}/dist/cli.js help`);
    t.true(result.stdout.includes(expectedOutput));
    result = await shell(`node ${CWD}/dist/cli.js h`);
    t.true(result.stdout.includes(expectedOutput));
});
ava_1.default.serial("$ memserver init | sets up the initial folder structure", async (t) => {
    t.plan(7);
    t.true(!fs_extra_1.default.pathExistsSync(`${CWD}/memserver`));
    const expectedOutput = "[Memserver CLI] /memserver/index.ts created\n" +
        "[Memserver CLI] /memserver/routes.ts created\n" +
        "[Memserver CLI] /memserver/initializer.ts created\n" +
        "[Memserver CLI] /memserver/fixtures folder created\n" +
        "[Memserver CLI] /memserver/models folder created\n";
    const { stdout } = await shell(`node ${CWD}/dist/cli.js init`);
    t.is(stdout, expectedOutput);
    const [indexBuffer, routesBuffer, initializerBuffer, fixturesFolderExistence, modelsFolderExists] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver/index.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/routes.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/initializer.ts`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/fixtures`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/models`)
    ]);
    const [targetIndexFileBuffer, targetRoutesFileBuffer, targetInitializerBuffer] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/index.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/routes.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/initializer.ts`)
    ]);
    t.is(indexBuffer.toString(), targetIndexFileBuffer.toString());
    t.is(routesBuffer.toString(), targetRoutesFileBuffer.toString());
    t.is(initializerBuffer.toString(), targetInitializerBuffer.toString());
    t.true(fixturesFolderExistence);
    t.true(modelsFolderExists);
});
ava_1.default.serial("$ memserver new | sets up the initial folder structure", async (t) => {
    t.plan(7);
    t.true(!fs_extra_1.default.pathExistsSync(`${CWD}/memserver`));
    const expectedOutput = "[Memserver CLI] /memserver/index.ts created\n" +
        "[Memserver CLI] /memserver/routes.ts created\n" +
        "[Memserver CLI] /memserver/initializer.ts created\n" +
        "[Memserver CLI] /memserver/fixtures folder created\n" +
        "[Memserver CLI] /memserver/models folder created\n";
    const { stdout } = await shell(`node ${CWD}/dist/cli.js new`);
    t.is(stdout, expectedOutput);
    const [indexBuffer, routesBuffer, initializerBuffer, fixturesFolderExistence, modelsFolderExists] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver/index.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/routes.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver/initializer.ts`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/fixtures`),
        fs_extra_1.default.pathExists(`${CWD}/memserver/models`)
    ]);
    const [targetIndexFileBuffer, targetRoutesFileBuffer, targetInitializerBuffer] = await Promise.all([
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/index.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/routes.ts`),
        fs_extra_1.default.readFile(`${CWD}/memserver-boilerplate/initializer.ts`)
    ]);
    t.is(indexBuffer.toString(), targetIndexFileBuffer.toString());
    t.is(routesBuffer.toString(), targetRoutesFileBuffer.toString());
    t.is(initializerBuffer.toString(), targetInitializerBuffer.toString());
    t.true(fixturesFolderExistence);
    t.true(modelsFolderExists);
});
ava_1.default.serial("$ memserver version | and $ memserver v", async (t) => {
    t.plan(2);
    let result = await shell(`node ${CWD}/dist/cli.js v`);
    t.is(result.stdout, `[Memserver CLI] ${require(`${CWD}/package.json`).version}\n`);
    result = await shell(`node ${CWD}/dist/cli.js version`);
    t.is(result.stdout, `[Memserver CLI] ${require(`${CWD}/package.json`).version}\n`);
});
