"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
ava_1.default.afterEach.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("MemServer require() should throw error if /memserver folder doesnt exist", async (t) => {
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const error = await t.throwsAsync(() => startMemserver(), {
        instanceOf: Error
    });
    t.true(/\/memserver folder doesn't exist for this directory!/.test(error.message));
});
ava_1.default.serial("MemServer require() should throw error if /memserver/models folder doesnt exist", async (t) => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const error = await t.throwsAsync(() => startMemserver(), {
        instanceOf: Error
    });
    t.true(/\/memserver\/models folder doesn't exist for this directory!/.test(error.message));
});
ava_1.default.serial("MemServer require() should throw error if /memserver/routes.ts doesnt exist", async (t) => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const error = await t.throwsAsync(() => startMemserver(), {
        instanceOf: Error
    });
    t.true(/\/memserver\/routes.ts doesn't exist for this directory!/.test(error.message));
});
ava_1.default.serial("MemServer require() should throw error if /memserver/initializer.ts doesnt exist", async (t) => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, "export default function() {}");
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const error = await t.throwsAsync(() => startMemserver(), {
        instanceOf: Error
    });
    t.true(/\/memserver\/initializer.ts doesn't exist for this directory!/.test(error.message));
});
ava_1.default.serial("MemServer require() exports a MemServer with right functions and empty DB when there is no model", async (t) => {
    t.plan(1);
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, "export default function() {}");
    await fs_extra_1.default.writeFile(`${CWD}/memserver/initializer.ts`, "export default function() {}");
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const Server = await startMemserver();
    t.true(!!Server.shutdown);
});
ava_1.default.serial("MemServer require() exports a MemServer with right functions and empty DB and models", async (t) => {
    t.plan(52);
    const modelFileContent = (fileName) => `import Model from '${CWD}/dist/model';
    export default class ${fileName} extends Model{
    }`;
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, modelFileContent("Photo")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/user.ts`, modelFileContent("User")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, modelFileContent("PhotoComment")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, "export default function() {}"),
        fs_extra_1.default.writeFile(`${CWD}/memserver/initializer.ts`, "export default function() {}")
    ]);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const User = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/user.ts`)))).default;
    const startMemserver = (await Promise.resolve().then(() => __importStar(require("../index")))).default;
    const Server = await startMemserver();
    t.deepEqual(Object.keys(Server), [
        "hosts",
        "handlers",
        "handledRequests",
        "passthroughRequests",
        "unhandledRequests",
        "requestReferences",
        "forcePassthrough",
        "disableUnhandled",
        "_nativeXMLHttpRequest",
        "running",
        "ctx",
        "_fetchProps",
        "_nativefetch",
        "_nativeHeaders",
        "_nativeRequest",
        "_nativeResponse",
        "Models",
        "handledRequest",
        "passthroughRequest",
        "unhandledRequest",
        "passthrough",
    ]);
    [Photo, PhotoComment, User].forEach((modelDefinition) => {
        t.deepEqual(modelDefinition.findAll(), []);
        [
            "name",
            "primaryKey",
            "defaultAttributes",
            "attributes",
            "count",
            "find",
            "findBy",
            "findAll",
            "insert",
            "update",
            "delete",
            "embed",
            "embedReferences",
            "serializer",
            "serialize",
            "getRelationship"
        ].forEach((method) => {
            t.true(method in modelDefinition);
        });
    });
});
