"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
const modelFileContent = (fileName) => `import Model from '${CWD}/dist/model';
export default class ${fileName} extends Model{
}`;
ava_1.default.before(async () => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, modelFileContent("Photo")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/user.ts`, modelFileContent("User")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
      import Photo from './models/photo';
      import Response from '../dist/response';

      export default function() {
        this.get('/photos', () => {
          const photos = Photo.findAll();

          return Response(202, { photos: Photo.serializer(photos) });
        });
      }`),
        fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)
    ]);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
      {
        id: 1,
        name: 'Ski trip',
        href: 'ski-trip.jpeg',
        is_public: false
      },
      {
        id: 2,
        name: 'Family photo',
        href: 'family-photo.jpeg',
        is_public: true
      },
      {
        id: 3,
        name: 'Selfie',
        href: 'selfie.jpeg',
        is_public: false
      }
    ];`),
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/users.ts`, `export default [
      {
        id: 1,
        first_name: 'Izel',
        last_name: 'Nakri'
      }
    ]`)
    ]);
});
ava_1.default.after.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.beforeEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.serial("MemServer.Response can be used outside the server file", async (t) => {
    t.plan(2);
    const User = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/user.ts`)))).default;
    const UserFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/users.ts`)))).default;
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Response = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/response`)))).default;
    UserFixtures.forEach((user) => User.insert(user));
    const Server = new MemServer();
    const $ = await Promise.resolve().then(() => __importStar(require("jquery")));
    Server.get("/users/:id", (request) => {
        const user = User.find(Number(request.params.id));
        if (user) {
            return Response(200, { user: User.serializer(user) });
        }
    });
    await $.ajax({
        type: "GET",
        url: "/users/1"
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { user: { id: 1, first_name: "Izel", last_name: "Nakri" } });
    });
});
ava_1.default.serial("MemServer.Response can be used inside the server file", async (t) => {
    t.plan(2);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Response = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/response`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    const $ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await $.getJSON("/photos", (data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 202);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
ava_1.default.serial("MemServer.Response can be used when overwriting an existing server route", async (t) => {
    t.plan(2);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Response = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/response`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    const $ = await Promise.resolve().then(() => __importStar(require("jquery")));
    Server.get("/photos", () => Response(500, { error: "Internal Server Error" }));
    try {
        await $.getJSON("/photos");
    }
    catch (jqXHR) {
        t.is(jqXHR.status, 500);
        t.deepEqual(jqXHR.responseJSON, { error: "Internal Server Error" });
    }
});
