"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const express_1 = __importDefault(require("express"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const mber_fastboot_express_middleware_1 = __importDefault(require("./test-helpers/fastboot-dist/mber-fastboot-express-middleware"));
const http_1 = __importDefault(require("http"));
const CWD = process.cwd();
const modelFileContent = (fileName) => `import Model from '${CWD}/dist/model';
export default class ${fileName} extends Model{
}`;
ava_1.default.beforeEach(async () => {
    await fs_extra_1.default.mkdirp(`${CWD}/memserver`);
    await Promise.all([fs_extra_1.default.mkdirp(`${CWD}/memserver/models`), fs_extra_1.default.mkdirp(`${CWD}/memserver/fixtures`)]);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, modelFileContent("Photo")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/user.ts`, modelFileContent("User")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, modelFileContent("PhotoComment")),
        fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, "export default function() {}"),
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
      {
        id: 1,
        name: 'Ski trip',
        href: 'ski-trip.jpeg',
        is_public: false
      },
      {
        id: 2,
        name: 'Family photo',
        href: 'family-photo.jpeg',
        is_public: true
      },
      {
        id: 3,
        name: 'Selfie',
        href: 'selfie.jpeg',
        is_public: false
      }
    ];`),
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photo-comments.ts`, `export default [
      {
        uuid: '499ec646-493f-4eea-b92e-e383d94182f4',
        content: 'What a nice photo!',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '77653ad3-47e4-4ec2-b49f-57ea36a627e7',
        content: 'Second photo',
        photo_id: 1,
        user_id: 2
      },
      {
        uuid: 'd351963d-e725-4092-a37c-1ca1823b57d3',
        content: 'Third photo',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '374c7f4a-85d6-429a-bf2a-0719525f5f29',
        content: 'Interesting indeed',
        photo_id: 2,
        user_id: 1
      }
    ];`)
    ]);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.afterEach.always(async () => {
    // NOTE: maybe remove require cache if needed
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("MemServer with JSDOM could be used with ember fastboot for server side rendering", async (t) => {
    const FASTBOOT_DIST_PATH = `${CWD}/src/test/test-helpers/fastboot-dist`;
    const jsdom = (await Promise.resolve().then(() => __importStar(require("jsdom")))).default;
    const FastBoot = (await Promise.resolve().then(() => __importStar(require("fastboot")))).default;
    const dom = new jsdom.JSDOM("<p>Hello</p>", { url: "http://localhost:3000" });
    global.window = dom.window;
    global.document = dom.window.document;
    global.self = dom.window.self;
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    global.fastboot = new FastBoot({
        distPath: FASTBOOT_DIST_PATH,
        resilient: true,
        shouldRender: true,
        sandboxGlobals: {
            global: global,
            self: global.self,
            window: global.window,
            document: global.document,
            location: global.window.location,
            XMLHttpRequest: global.window.XMLHttpRequest,
            $: window.$,
            jQuery: window.$,
            navigator: global.window.navigator
        }
    });
    const server = express_1.default();
    server.use("/assets", express_1.default.static(`${FASTBOOT_DIST_PATH}/assets`));
    server.use(express_1.default.static(FASTBOOT_DIST_PATH));
    server.use((req, res, next) => {
        const fastbootByPassQueryParam = req.query.fastboot && req.query.fastboot === "false";
        if (fastbootByPassQueryParam) {
            return res.sendFile(`${FASTBOOT_DIST_PATH}/index.html`);
        }
        const middleware = mber_fastboot_express_middleware_1.default({
            distPath: FASTBOOT_DIST_PATH,
            fastboot: global.fastboot,
            resilient: true,
            shouldRender: true
        });
        return middleware(req, res, next);
    });
    server.listen(5000, () => console.log("server listening on port 5000"));
    const response = await makeHTTPRequest("http://localhost:5000");
    console.log("response is", response);
    // TODO: assert against the response
    t.true(true);
    // TODO: BOOT UP puppeteer and check without ?fastboot=false the content
    // server.close();
});
function makeHTTPRequest(url) {
    return new Promise((resolve, reject) => {
        return http_1.default
            .get(url, (res) => {
            const { statusCode } = res;
            if (statusCode !== 200) {
                console.log("statusCode is", statusCode);
                console.log("response is");
                console.log(res);
                throw new Error(`Request Failed.\nStatus Code: ${statusCode}`);
            }
            res.setEncoding("utf8");
            let rawData = "";
            res.on("data", (chunk) => {
                rawData += chunk;
            });
            res.on("end", () => resolve(rawData));
        })
            .on("error", (error) => reject(`http error: ${error.message}`));
    });
}
// NOTE: mber tests wait 1s from JSDOM, but not needed here
