"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
// TODO: test reset() function, if implementation is required
ava_1.default.before(async () => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, `
      import Model from '${CWD}/dist/model';

      export default class Photo extends Model {
        static defaultAttributes = {
          is_public: true,
          name() {
            return 'Imported photo';
          }
        }
        static publicPhotos() {
          return super.findAll({ is_public: true });
        }
      }
    `),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, `
      import Model from '${CWD}/dist/model';

      export default class PhotoComment extends Model {
        static defaultAttributes = {
          inserted_at() {
            return new Date().toJSON();
          },
          is_important: true
        }
        static forPhoto(photo) {
          return super.findAll({ photo_id: photo.id });
        }
      }
    `),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/user.ts`, `
      import Model from '${CWD}/dist/model';

      export default class User extends Model {
      }
    `),
        fs_extra_1.default.writeFile(`${CWD}/memserver/server.js`, "export default function() {}"),
        fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)
    ]);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
      {
        id: 1,
        name: 'Ski trip',
        href: 'ski-trip.jpeg',
        is_public: false
      },
      {
        id: 2,
        name: 'Family photo',
        href: 'family-photo.jpeg',
        is_public: true
      },
      {
        id: 3,
        name: 'Selfie',
        href: 'selfie.jpeg',
        is_public: false
      }
    ];`),
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photo-comments.ts`, `export default [
      {
        uuid: '499ec646-493f-4eea-b92e-e383d94182f4',
        content: 'What a nice photo!',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '77653ad3-47e4-4ec2-b49f-57ea36a627e7',
        content: 'I agree',
        photo_id: 1,
        user_id: 2
      },
      {
        uuid: 'd351963d-e725-4092-a37c-1ca1823b57d3',
        content: 'I was kidding',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '374c7f4a-85d6-429a-bf2a-0719525f5f29',
        content: 'Interesting indeed',
        photo_id: 2,
        user_id: 1
      }
    ];`)
    ]);
});
ava_1.default.beforeEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.afterEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.after.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("$Model.name gets set correctly", async (t) => {
    t.plan(2);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    t.is(Photo.name, "Photo");
    t.is(PhotoComment.name, "PhotoComment");
});
ava_1.default.serial("$Model.primaryKey gets set correctly", async (t) => {
    t.plan(7);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const User = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/user.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    t.is(Photo.primaryKey, null);
    t.is(PhotoComment.primaryKey, null);
    t.is(User.primaryKey, null);
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    t.is(Photo.primaryKey, "id");
    t.is(PhotoComment.primaryKey, "uuid");
    t.is(User.primaryKey, null);
    t.true(Photo._DB === PhotoComment._DB);
});
ava_1.default.serial("$Model.defaultAttributes gets set correctly", async (t) => {
    t.plan(10);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, `
      import Model from '${CWD}/dist/model';

      export default class Photo extends Model {
        static defaultAttributes = {
          is_public: true,
          name() {
            return 'Imported photo';
          }
        }
        static publicPhotos() {
          return super.findAll({ is_public: true });
        }
      }
    `),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, `
      import Model from '${CWD}/dist/model';

      export default class PhotoComment extends Model {
        static defaultAttributes = {
          inserted_at() {
            return new Date().toJSON();
          },
          is_important: true
        }
        static forPhoto(photo) {
          return super.findAll({ photo_id: photo.id });
        }
      }
    `)
    ]);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const User = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/user.ts`)))).default;
    const initialPhotoDefaultAttributes = Photo.defaultAttributes;
    const initialPhotoCommentDefaultAttributes = PhotoComment.defaultAttributes;
    t.deepEqual(Object.keys(initialPhotoDefaultAttributes), ["is_public", "name"]);
    t.is(initialPhotoDefaultAttributes.is_public, true);
    t.true(initialPhotoDefaultAttributes.name.toString().includes("Imported photo"));
    t.deepEqual(Object.keys(initialPhotoCommentDefaultAttributes), ["inserted_at", "is_important"]);
    t.true(initialPhotoCommentDefaultAttributes.inserted_at.toString().includes(".toJSON();"));
    t.is(initialPhotoCommentDefaultAttributes.is_important, true);
    t.deepEqual(User.defaultAttributes, {});
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    t.is(Photo.defaultAttributes, initialPhotoDefaultAttributes);
    t.deepEqual(PhotoComment.defaultAttributes, initialPhotoCommentDefaultAttributes);
    t.deepEqual(User.defaultAttributes, {});
});
ava_1.default.serial("$Model.attributes gets set correctly", async (t) => {
    t.plan(6);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const User = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/user.ts`)))).default;
    t.deepEqual(Photo.attributes, ["is_public", "name"]);
    t.deepEqual(PhotoComment.attributes, ["inserted_at", "is_important"]);
    t.deepEqual(User.attributes, []);
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    t.deepEqual(Photo.attributes, ["is_public", "name", "id", "href"]);
    t.deepEqual(PhotoComment.attributes, [
        "inserted_at",
        "is_important",
        "uuid",
        "content",
        "photo_id",
        "user_id"
    ]);
    t.deepEqual(User.attributes, []);
});
ava_1.default.serial("$Model.count counts the models correctly", async (t) => {
    t.plan(4);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    t.is(Photo.count(), 0);
    t.is(PhotoComment.count(), 0);
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    t.is(Photo.count(), 3);
    t.is(PhotoComment.count(), 4);
});
ava_1.default.serial("$Model can have custom methods/queries for the model", async (t) => {
    t.plan(2);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoComment = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo-comment.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    const PhotoCommentFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photo-comments.ts`))))
        .default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    PhotoCommentFixtures.forEach((photoComment) => PhotoComment.insert(photoComment));
    const photo = Photo.find(1);
    t.deepEqual(PhotoComment.forPhoto(photo), PhotoComment.findAll({ photo_id: photo.id }));
    t.deepEqual(Photo.publicPhotos(), Photo.findAll({ is_public: true }));
});
// test.serial("$Model.resetDatabase() resets the models DB", async (t) => {});
// maybe add modelNames, and fixtures
