"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
ava_1.default.before(async () => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, `
      import Model from '${CWD}/dist/model';

      export default class Photo extends Model {
        static defaultAttributes = {
          is_public: true,
          name() {
            return 'Some default name';
          }
        }

      }
    `),
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo-comment.ts`, `
      import Model from '${CWD}/dist/model';

      export default class PhotoComment extends Model {
        static defaultAttributes = {
          inserted_at() {
            return '2017-10-25T20:54:04.447Z';
          },
          is_important: true
        }
      }
    `),
        fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)
    ]);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
      {
        id: 1,
        name: 'Ski trip',
        href: 'ski-trip.jpeg',
        is_public: false,
        user_id: 1
      },
      {
        id: 2,
        name: 'Family photo',
        href: 'family-photo.jpeg',
        is_public: true,
        user_id: 1
      },
      {
        id: 3,
        name: 'Selfie',
        href: 'selfie.jpeg',
        is_public: false,
        user_id: 1
      }
    ];`),
        fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photo-comments.ts`, `export default [
      {
        uuid: '499ec646-493f-4eea-b92e-e383d94182f4',
        content: 'What a nice photo!',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '77653ad3-47e4-4ec2-b49f-57ea36a627e7',
        content: 'I agree',
        photo_id: 1,
        user_id: 2
      },
      {
        uuid: 'd351963d-e725-4092-a37c-1ca1823b57d3',
        content: 'I was kidding',
        photo_id: 1,
        user_id: 1
      },
      {
        uuid: '374c7f4a-85d6-429a-bf2a-0719525f5f29',
        content: 'Interesting indeed',
        photo_id: 2,
        user_id: 1
      }
    ];`)
    ]);
});
ava_1.default.beforeEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.after.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("namespace configuration option could be passed in during MemServer.start()", async (t) => {
    t.plan(2);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from '${CWD}/memserver/models/photo';
    import Response from '../dist/response';

    export default function() {
      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        namespace: "api/v1",
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/api/v1/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
ava_1.default.serial("server this.namespace() configuration can overwrite existing namespace config", async (t) => {
    t.plan(2);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from './models/photo';
    import Response from '../dist/response';

    export default function() {
      this.namespace = 'api/';

      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });
    }
  `);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        namespace: "api/v1",
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/api/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
ava_1.default.serial("urlPrefix configuration option could be passed in during MemServer.start()", async (t) => {
    t.plan(2);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/server.ts`, `
    import Photo from './models/photo';
    import Response from '../dist/response';

    export default function() {
      this.namespace = 'api/';
      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });
    }
  `);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        urlPrefix: "http://twitter.com",
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "http://twitter.com/api/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
ava_1.default.serial("server this.urlPrefix() configuration can overwrite existing urlPrefix config", async (t) => {
    t.plan(2);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from './models/photo';
    import Response from '../dist/response';

    export default function() {
      this.urlPrefix = 'http://facebook.com/';
      this.namespace = 'api';

      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        urlPrefix: "http://twitter.com",
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "http://facebook.com/api/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
ava_1.default.serial('timing configuration option could be passed in during MemServer.start()', async (t) => {
    t.plan(3);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from './models/photo';
    import Response from '../dist/response';

    export default function() {
      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });
    }
  `);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        timing: 3000,
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    let ThreeSecondsPassed = false;
    setTimeout(() => { ThreeSecondsPassed = true; }, 2200);
    await window.$.ajax({
        type: 'GET', url: '/photos', headers: { 'Content-Type': 'application/json' }
    }).then((data, textStatus, jqXHR) => {
        t.true(ThreeSecondsPassed);
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
    });
});
// test.serial('server this.get(url, timing) configuration can overwrite existing timing config', async () => {
//
// });
