"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const CWD = process.cwd();
ava_1.default.before(async () => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, `
      import Model from '${CWD}/dist/model';

      export default class Photo extends Model {
        static defaultAttributes = {
          is_public: true,
          name() {
            return 'Some default name';
          }
        }
      }
    `),
        fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)
    ]);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
    {
      id: 1,
      name: 'Ski trip',
      href: 'ski-trip.jpeg',
      is_public: false,
      user_id: 1
    },
    {
      id: 2,
      name: 'Family photo',
      href: 'family-photo.jpeg',
      is_public: true,
      user_id: 1
    },
    {
      id: 3,
      name: 'Selfie',
      href: 'selfie.jpeg',
      is_public: false,
      user_id: 1
    }
  ];`);
});
ava_1.default.beforeEach(() => {
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.after.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
});
ava_1.default.serial("POST /resources work with shortcut", async (t) => {
    t.plan(5);
    await writeServerFileWithNoHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.count(), 3);
    await window.$.ajax({
        type: "POST",
        url: "/photos",
        headers: { "Content-Type": "application/json" },
        data: JSON.stringify({ photo: { name: "Izel Nakri" } })
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 201);
        t.deepEqual(data, { photo: Photo.serializer(Photo.find(4)) });
        t.is(Photo.count(), 4);
        t.deepEqual(Photo.find(4), {
            id: 4,
            name: "Izel Nakri",
            is_public: true,
            href: undefined,
            user_id: undefined
        });
    });
});
ava_1.default.serial("GET /resources works with shortcut", async (t) => {
    t.plan(4);
    await writeServerFileWithNoHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.count(), 3);
    await window.$.ajax({
        type: "GET",
        url: "/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
        t.is(Photo.count(), 3);
    });
});
ava_1.default.serial("GET /resources/:id works with shortcut", async (t) => {
    t.plan(2);
    await writeServerFileWithNoHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photo: Photo.serializer(Photo.find(1)) });
    });
});
ava_1.default.serial("PUT /resources/:id works with shortcut", async (t) => {
    t.plan(4);
    await writeServerFileWithNoHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.find(1).name, "Ski trip");
    await window.$.ajax({
        type: "PUT",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" },
        data: JSON.stringify({ photo: { id: 1, name: "New custom title" } })
    }).then((data, textStatus, jqXHR) => {
        const photo = Photo.find(1);
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photo: Photo.serializer(photo) });
        t.is(photo.name, "New custom title");
    });
});
ava_1.default.serial("DELETE /resources/:id works with shortcut", async (t) => {
    t.plan(5);
    await writeServerFileWithNoHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.count(), 3);
    await window.$.ajax({
        type: "DELETE",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 204);
        t.deepEqual(data, undefined);
        t.is(Photo.count(), 2);
        t.is(Photo.find(1), undefined);
    });
});
ava_1.default.serial("throws an helpful error message when shortcuts model is not found", async (t) => {
    t.plan(2);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    export default function(Models) {
      this.post('/photos');
      this.get('/photos');
      this.get('/photos/:id');
      this.put('/photos/:id');
      this.delete('/photos/:id');

      this.get('/houses');
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const routes = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default;
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    const error = t.throws(() => new MemServer({ routes }), { instanceOf: Error });
    t.true(/\[Memserver\] GET \/houses route handler cannot be generated automatically: House is not on your window.House, also please check that your route name matches the model reference or create a custom handler function/.test(error.message));
});
ava_1.default.serial("GET /houses could use Photo model for its default route generation", async (t) => {
    t.plan(4);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from './models/photo';

    export default function(Models) {
      this.post('/photos');
      this.get('/photos');
      this.get('/photos/:id');
      this.put('/photos/:id');
      this.delete('/photos/:id');

      this.get('/houses', Photo);
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const routes = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default;
    new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.count(), 3);
    await window.$.ajax({
        type: "GET",
        url: "/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(data, { photos: Photo.serializer(Photo.findAll()) });
        t.is(Photo.count(), 3);
    });
});
ava_1.default.serial("POST /resources works correctly with undefined handler response", async (t) => {
    t.plan(4);
    await writeServerFileWithEmptyHandlers();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    t.is(Photo.count(), 3);
    await window.$.ajax({
        type: "POST",
        url: "/photos",
        headers: { "Content-Type": "application/json" }
    }).catch((jqXHR) => {
        t.is(jqXHR.status, 500);
        t.deepEqual(jqXHR.responseJSON, {
            error: "[Memserver] POST /photos route handler did not return anything to respond to the request!"
        });
        t.is(Photo.count(), 3);
    });
});
ava_1.default.serial("GET /resources works correctly with undefined handler response", async (t) => {
    t.plan(2);
    await writeServerFileWithEmptyHandlers();
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/photos",
        headers: { "Content-Type": "application/json" }
    }).catch((jqXHR) => {
        t.is(jqXHR.status, 500);
        t.deepEqual(jqXHR.responseJSON, {
            error: "[Memserver] GET /photos route handler did not return anything to respond to the request!"
        });
    });
});
ava_1.default.serial("GET /resources/:id works correctly with undefined handler response", async (t) => {
    t.plan(2);
    await writeServerFileWithEmptyHandlers();
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" }
    }).catch((jqXHR) => {
        t.is(jqXHR.status, 500);
        t.deepEqual(jqXHR.responseJSON, {
            error: "[Memserver] GET /photos/1 route handler did not return anything to respond to the request!"
        });
    });
});
ava_1.default.serial("PUT /resources/:id works correctly with undefined handler response", async (t) => {
    t.plan(2);
    await writeServerFileWithEmptyHandlers();
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "PUT",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" },
        data: JSON.stringify({ photo: { id: 1, name: "New Name" } })
    }).catch((jqXHR) => {
        t.is(jqXHR.status, 500);
        t.deepEqual(jqXHR.responseJSON, {
            error: "[Memserver] PUT /photos/1 route handler did not return anything to respond to the request!"
        });
    });
});
ava_1.default.serial("DELETE /resources/:id works correctly with undefined handler response", async (t) => {
    t.plan(2);
    await writeServerFileWithEmptyHandlers();
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "DELETE",
        url: "/photos/1",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 204);
        t.deepEqual(jqXHR.responseJSON, {});
    });
});
async function writeServerFileWithNoHandlers() {
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    export default function() {
      this.post('/photos');
      this.get('/photos');
      this.get('/photos/:id');
      this.put('/photos/:id');
      this.delete('/photos/:id');
    }
  `);
}
async function writeServerFileWithEmptyHandlers() {
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    export default function() {
      this.post('/photos', () => {});
      this.get('/photos', () => {});
      this.get('/photos/:id', () => {});
      this.put('/photos/:id', () => {});
      this.delete('/photos/:id', () => {});
    }
  `);
}
