"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const sinon_1 = __importDefault(require("sinon"));
let actualServer;
const AUTHENTICATION_TOKEN = "ec25fc7b-6ee2-4bda-b57c-6c9867b30ff4";
const AJAX_AUTHORIZATION_HEADERS = {
    "Content-Type": "application/json",
    Authorization: `Token ${AUTHENTICATION_TOKEN}`
};
const CWD = process.cwd();
ava_1.default.before(async () => {
    await fs_extra_1.default.mkdir(`${CWD}/memserver`);
    await fs_extra_1.default.mkdir(`${CWD}/memserver/models`);
    await Promise.all([
        fs_extra_1.default.writeFile(`${CWD}/memserver/models/photo.ts`, `
    import Model from '${CWD}/dist/model';

    export default class Photo extends Model {
    }`),
        fs_extra_1.default.mkdir(`${CWD}/memserver/fixtures`)
    ]);
    await fs_extra_1.default.writeFile(`${CWD}/memserver/fixtures/photos.ts`, `export default [
    {
      id: 1,
      name: 'Ski trip',
      href: 'ski-trip.jpeg',
      is_public: false,
      user_id: 1
    },
    {
      id: 2,
      name: 'Family photo',
      href: 'family-photo.jpeg',
      is_public: true,
      user_id: 1
    },
    {
      id: 3,
      name: 'Selfie',
      href: 'selfie.jpeg',
      is_public: false,
      user_id: 1
    }
  ];`);
    const app = express_1.default();
    app.use(cors_1.default());
    app.get("/films", (req, res) => {
        res.json({ film: "responsed correctly" });
    });
    app.get("/movies/too-big-to-fail", (req, res) => {
        res.json({ movie: "is too-big-to-fail" });
    });
    actualServer = app.listen(4000, () => console.log("Web server started on port 4000"));
});
ava_1.default.beforeEach(async () => {
    await fs_extra_1.default.writeFile(`${CWD}/memserver/routes.ts`, `
    import Photo from './models/photo';
    import Response from '../dist/response';

    export default function() {
      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });

      this.passthrough('/films');
      this.passthrough('http://localhost:4000/films');
      this.passthrough('http://localhost:4000/movies/*');
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
});
ava_1.default.after.always(async () => {
    if (await fs_extra_1.default.pathExists(`${CWD}/memserver`)) {
        await fs_extra_1.default.remove(`${CWD}/memserver`);
    }
    actualServer.close();
});
ava_1.default.serial("[MemServer.Server] throws an error when MemServer tried to intercept an undeclared route", async (t) => {
    t.plan(1);
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    Server.unhandledRequest = sinon_1.default.spy();
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    window.$.ajax({
        type: "GET",
        url: "/izelnakri",
        headers: { "Content-Type": "application/json" }
    });
    t.true(Server.unhandledRequest.calledOnce, "Server.unhandledRequest called once");
});
ava_1.default.serial("[MemServer.Server] this.passthrough(url) shortcut works", async (t) => {
    t.plan(2);
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "http://localhost:4000/films",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(jqXHR.responseJSON, { film: "responsed correctly" });
    });
});
ava_1.default.serial("[MemServer.Server] this.passthrough(url) shortcut works with wild cards", async (t) => {
    t.plan(2);
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "http://localhost:4000/movies/too-big-to-fail",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(jqXHR.responseJSON, { movie: "is too-big-to-fail" });
    });
});
ava_1.default.serial("[MemServer.Server] can create global passthrough via this.passthrough()", async (t) => {
    t.plan(6);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
    await writeGlobalPassthroughServerFile();
    const Photo = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/models/photo.ts`)))).default;
    const PhotoFixtures = (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/fixtures/photos.ts`)))).default;
    PhotoFixtures.forEach((photo) => Photo.insert(photo));
    await (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/setup-dom`)))).default();
    const MemServer = (await Promise.resolve().then(() => __importStar(require(`${CWD}/dist/server`)))).default;
    const Server = new MemServer({
        routes: (await Promise.resolve().then(() => __importStar(require(`${CWD}/memserver/routes`)))).default
    });
    Server.unhandledRequest = sinon_1.default.spy();
    window.$ = await Promise.resolve().then(() => __importStar(require("jquery")));
    await window.$.ajax({
        type: "GET",
        url: "/photos",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(jqXHR.responseJSON, { photos: Photo.serializer(Photo.findAll()) });
    });
    await window.$.ajax({
        type: "GET",
        url: "http://localhost:4000/films",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(jqXHR.responseJSON, { film: "responsed correctly" });
    });
    await window.$.ajax({
        type: "GET",
        url: "http://localhost:4000/movies/too-big-to-fail",
        headers: { "Content-Type": "application/json" }
    }).then((data, textStatus, jqXHR) => {
        t.is(jqXHR.status, 200);
        t.deepEqual(jqXHR.responseJSON, { movie: "is too-big-to-fail" });
    });
});
// TODO: TEST BELOW ISNT WORKING: has beforeEach!! afterwards
// TODO: test this.passthrough('/something') when there is this.namespace;
// NOTE: passthrough order? investigate
async function writeGlobalPassthroughServerFile() {
    await fs_extra_1.default.writeFile(`${CWD}/memserver/server.js`, `
    import Response from '../lib/response';

    export default function({ Photo }) {
      this.get('/photos', () => {
        const photos = Photo.findAll();

        if (!photos || photos.length === 0) {
          return Response(404, { error: 'Not found' });
        }

        return { photos: Photo.serializer(photos) };
      });

      this.passthrough();
    }
  `);
    Object.keys(require.cache).forEach((key) => delete require.cache[key]);
}
